package evaluation;

import java.util.LinkedList;

import evaluation.ui.SubQuestionPanel;

public class Answer {
	public static int INVALID = -1;

	private String name = null;

	private int presentationNum = INVALID;

	private int questionNum = INVALID;

	private int questionAnswer = INVALID;

	private int correctAnswer = INVALID;

	private Question question = null;

	public Answer() {

	}

	/**
	 * Construct SUBJECTIVE answer from a str
	 */
	public Answer(String str, Judge curJudge) {
		presentationNum = curJudge.getCurPresentationNumber();

		questionNum = getQuestionNum(str);
		questionAnswer = getQuestionAnswer(str);
		name = curJudge.getName();
	}

	public Answer(String str, LinkedList questions, Judge judge) {
		presentationNum = judge.getCurPresentationNumber();

		questionNum = getQuestionNum(str);
		questionAnswer = getQuestionAnswer(str);
		correctAnswer = ((Question) questions.get(questionNum))
				.getCorrectAnswer();
		question = (Question) questions.get(questionNum);
		name = judge.getName();
	}

	public int getPresentationNum() {
		return presentationNum;
	}

	public int getQuestionNum() {
		return questionNum;
	}

	public int getQuestionAnswer() {
		return questionAnswer;
	}

	public int getCorrectAnswer() {
		return correctAnswer;
	}

	public void setPresentationNum(int newPresentationNum) {
		presentationNum = newPresentationNum;
	}

	public void setQuestionNum(int newQuestionNum) {
		questionNum = newQuestionNum;
	}

	public void setQuestionAnswer(int newQuestionAnswer) {
		questionAnswer = newQuestionAnswer;
	}

	public void setCorrectAnswer(int newCorrectAnswer) {
		correctAnswer = newCorrectAnswer;
	}

	public String toString() {
		String result = "";
		result += "Name: " + name + "\n";
		result += "Presentation number: " + presentationNum + "\n";
		result += "Question number: " + questionNum + "\n";
		result += "Question answer: " + questionAnswer + "\n";
		result += "Correct answer: " + correctAnswer + "\n";
		return result;
	}

	/**
	 * Get the question number from the str The format of str is
	 * "questionNum@questionAnswer"
	 */
	private int getQuestionNum(String str) {
		int index = str.indexOf(SubQuestionPanel.splitString);
		String preNum = str.substring(0, index);
		Integer preInt = new Integer(preNum);
		return preInt.intValue();
	}

	/**
	 * Get the question answer from the str. The format of str is
	 * "questionNum@questionAnswer"
	 */
	private int getQuestionAnswer(String str) {
		int index = str.indexOf(SubQuestionPanel.splitString);
		String answerNum = str.substring(index + 1, str.length());
		Integer answerInt = new Integer(answerNum);
		return answerInt.intValue();
	}

	public String toResultString() {
		String result = "";
		if (question != null) {
			result += "Question:" + question.getQuestionText() + "\n";
		} else {
			result += "Question number: " + questionNum + "\n";
		}
		result += "Question answer: " + questionAnswer + "\n";
		if (correctAnswer != INVALID) {
			result += "Correct answer: " + correctAnswer + "\n";
		}
		return result;
	}

}
