package evaluation;

import java.io.FileWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import evaluation.models.EvaluationModel;
import evaluation.models.ObjEvaluationModel;

public class Judge {

	private int curPresentationNum = Answer.INVALID;

	private String curPresentationName = null;

	private String name = "";

	// The map containing presentation number and corresponding answers
	private HashMap presentationAnswers = new HashMap();

	// The list of current answers
	private LinkedList curAnswers = new LinkedList();

	/**
	 * @param curPresentationNum
	 *            The curPresentationNum to set.
	 */
	public void setCurPresentationNumber(int presentationNum) {
		storeCurPresentation();
		curAnswers = new LinkedList();

		if (presentationAnswers.containsKey(new Integer(presentationNum))) {
			curAnswers = (LinkedList) presentationAnswers.get(new Integer(
					presentationNum));
		}
		this.curPresentationNum = presentationNum;
	}
	
	public void storeCurPresentation() {
		if (curPresentationNum != Answer.INVALID) {

			presentationAnswers
					.put(new Integer(curPresentationNum), curAnswers);
		}
	}

	public int getCurPresentationNumber() {
		return curPresentationNum;
	}

	public Judge(String newName) {
		name = newName;
	}

	public String getName() {
		return name;
	}

	public LinkedList getAnswers() {
		return curAnswers;
	}

	/**
	 * Check all the answers and get the answer for current question
	 */
	public Answer getAnswerForQuestion(int questionNum) {
		for (int i = 0; i < curAnswers.size(); i++) {
			Answer curAnswer = (Answer) curAnswers.get(i);
			if (curAnswer.getPresentationNum() == curPresentationNum
					&& curAnswer.getQuestionNum() == questionNum) {
				return curAnswer;
			}
		}
		return null;
	}

	/**
	 * Add answer according to the str.
	 */
	public void addAnswer(String str) {
		Answer newAnswer = new Answer(str, this);
		for (int i = 0; i < curAnswers.size(); i++) {
			if (sameQuestion(newAnswer, (Answer) curAnswers.get(i))) {
				curAnswers.remove(i);
				i--;
			}
		}
		curAnswers.add(newAnswer);
		// System.out.println("Adding:\n " + newAnswer);
	}

	public void addAnswer(Answer newAnswer) {
		curAnswers.add(newAnswer);
	}

	public boolean sameQuestion(Answer answer1, Answer answer2) {
		if (answer1.getPresentationNum() == answer2.getPresentationNum()
				&& answer1.getQuestionNum() == answer2.getQuestionNum()) {
			return true;
		} else {
			return false;
		}
	}

	public static void main(String[] argv) throws Exception {
	}

	public void outputAllAnswers(LinkedList fileList, EvaluationModel model) throws Exception{
		String type = "";
		if (model instanceof ObjEvaluationModel) {
			type = "obj";
		} else {
			type = "sub";
		}
		
		FileWriter fw=new FileWriter(".\\results\\" + name + "-" + type + ".txt");


		storeCurPresentation();
		Iterator it = presentationAnswers.keySet().iterator();
		while (it.hasNext()) {
			Integer curNum = (Integer) it.next();

			String fileName = fileList.get(curNum.intValue()).toString();
			LinkedList answers = (LinkedList) presentationAnswers.get(curNum);
			Iterator it1 = answers.iterator();
			while (it1.hasNext()) {
				fw.write("File:" + fileName + "\n");
				fw.write(((Answer)it1.next()).toResultString() + "\n");
			}
		}
		fw.close();
	}

	public void storeAnswers() {
		presentationAnswers.put(new Integer(curPresentationNum), curAnswers);
	}

	public void addAnswer(String str, LinkedList questions) {

		Answer newAnswer = new Answer(str, questions, this);
		for (int i = 0; i < curAnswers.size(); i++) {
			if (sameQuestion(newAnswer, (Answer) curAnswers.get(i))) {
				curAnswers.remove(i);
				i--;
			}
		}
		curAnswers.add(newAnswer);
		// System.out.println("Adding: " + newAnswer);

	}

	public String getCurPresentationName() {
		return curPresentationName;
	}

	public void setCurPresentationName(String curPresentationName) {
		this.curPresentationName = curPresentationName;
	}

}
