package evaluation;

import java.util.LinkedList;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import evaluation.utils.XMLUtils;

public class Question {

	private static final String questionStr = "Question";

	private static final String optionsStr = "Options";
	
	private static final String fileNameStr = "File";
	
	private static final String correctAnswerStr = "CorrectAnswer";
		
	
	private static final String textStr = "Text";

	private static final String optionSplitChar = ",";

	// The number of options
	public static int INVALID = 5;

	public static String questionPreStr = "What is the value of ";

	public static String questionEndStr = "?";

	public static String noIdeaStr = "No idea";

	public static String noInfoStr = "The information is not provided";

	private String questionText = "";

	private LinkedList optionTexts = new LinkedList();
	
	private int correctAnswer = INVALID;

	private String fileName = null;
	
	/**
	 * @return Returns the correctAnswer.
	 */
	public int getCorrectAnswer() {
		return correctAnswer;
	}


	/**
	 * @param correctAnswer The correctAnswer to set.
	 */
	public void setCorrectAnswer(int correctAnswer) {
		this.correctAnswer = correctAnswer;
	}


	/**
	 * @return Returns the optionTexts.
	 */
	public LinkedList getOptionTexts() {
		return optionTexts;
	}


	/**
	 * @param optionTexts The optionTexts to set.
	 */
	public void setOptionTexts(LinkedList optionTexts) {
		this.optionTexts = optionTexts;
	}


	/**
	 * @return Returns the questionText.
	 */
	public String getQuestionText() {
		return questionText;
	}


	/**
	 * @param questionText The questionText to set.
	 */
	public void setQuestionText(String questionText) {
		this.questionText = questionText;
	}


	public static Question fromDomNode(Node node) throws Exception {
		Question question = null;
		int type = node.getNodeType();

		switch (type) {
		case Node.DOCUMENT_NODE: {
			throw new Exception("Bad model file format:" + node.toString());
		}
		case Node.ELEMENT_NODE: {

			String name = node.getNodeName();

			if (name.equalsIgnoreCase(questionStr)) {
				question = new Question();
				if (node.hasChildNodes()) {
					NodeList children = node.getChildNodes();
					for (int i = 0; i < children.getLength(); i++) {
						Node curNode = children.item(i);
						String curName = curNode.getNodeName();
						String text = XMLUtils.getText(curNode);
						if (curName.equalsIgnoreCase(textStr)) {
							question.setQuestionText(text);
						} else if (curName.equalsIgnoreCase(optionsStr)) {
							question.setOptionTexts(optionsFromText(text));
						} else if (curName.equalsIgnoreCase(fileNameStr)) {
							question.setFileName(text);
						} else if (curName.equalsIgnoreCase(correctAnswerStr)) {
							int correctAnswer = Integer.parseInt(text);
							question.setCorrectAnswer(correctAnswer);
						}
					}

				}
			} 
		}
		}

		return question;
	}


	private static LinkedList optionsFromText(String optionsText) {
		String[] options = optionsText.split(optionSplitChar);
		LinkedList result = new LinkedList();
		for (int i = 0; i < options.length; i++) {
			result.add(options[i]);
		}
		return result;
	}


	public String getFileName() {
		return fileName;
	}


	public void setFileName(String fileName) {
		this.fileName = fileName;
	}



}
