package evaluation.models;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import evaluation.Judge;

public abstract class EvaluationModel {
	private static final String modelsStr = "models";

	private static final String subEvalStr = "SubjectiveEvaluation";
	
	private static final String objEvalStr = "ObjectiveEvaluation";
	
	protected static final String pathStr = "Path";
	
	protected static final String questionsStr = "Questions";
	
	protected static final String nameStr = "Name";
	
	protected static final String instructionStr = "Instruction";

	public String instruction = "No instruction available.";
	
	protected String name = null;

	protected String dataPath = null;

	protected Judge judge = null;
	
	protected LinkedList questions = new LinkedList();

	// The number of current evaluation set
	protected int curSetNum = -1;

	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}


	public abstract void startEvaluation();

	public void endEvaluation(LinkedList fileList) {
		try {
			judge.outputAllAnswers(fileList, this);
			System.exit(0);
		} catch (Exception e) {
			JOptionPane.showConfirmDialog(null, "Can't write to result file.");
			System.exit(-1);
		}

	}

	public static Judge getCurrentJudge() {
		String inputValue = "";
		while (inputValue.length() == 0) {
			inputValue = JOptionPane.showInputDialog(null,
					"Please input your name");
			if (inputValue == null) {
				System.exit(-1);
			}
		}
		return new Judge(inputValue);
	}

	public Judge getJudge() {
		return judge;
	}

	public void setJudge(Judge judge) {
		this.judge = judge;
	}

	public String getDataPath() {
		return dataPath;
	}

	public void setDataPath(String dataPath) {
		this.dataPath = dataPath;
	}

	/**
	 * Read in a list of models from the models file
	 * 
	 * @param modelsFile
	 * @return
	 */
	public static List modelsFromFile(String modelsFile) {
		ArrayList models = new ArrayList();
		Document doc = null;
		try {
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			doc = db.parse(modelsFile);
			if (doc == null) {
				throw new Exception("Bad model file format:" + doc);
			}
			Node modelsNode = null;
			// Get the element node from children of doc
			if (doc.hasChildNodes()) {
				NodeList children = doc.getChildNodes();
				for (int i = 0; i < children.getLength(); i++) {
					Node curNode = children.item(i);
					int type = curNode.getNodeType();
					if (type == Node.ELEMENT_NODE) {
						modelsNode = curNode;
						break;
					}
				}

			} else {
				throw new Exception("Bad model file format:"
						+ doc.getNodeName());
			}
			if (modelsNode == null) {
				throw new Exception("Bad model file format:"
						+ doc.getNodeName());
			}

			String name = modelsNode.getNodeName();
			if (!name.equalsIgnoreCase(modelsStr)) {
				throw new Exception("Bad model file format:"
						+ modelsNode.getNodeName());
			}

			if (modelsNode.hasChildNodes()) {
				NodeList children = modelsNode.getChildNodes();
				for (int i = 0; i < children.getLength(); i++) {
					try {
						EvaluationModel curModel = EvaluationModel
								.fromDomNode(children.item(i));
						if (curModel != null) {
							models.add(curModel);
						}
					} catch (Exception e) {
						e.printStackTrace();
					}
				}

			}

		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, e.getMessage());
			System.exit(0);
		}
		return models;
	}

	/**
	 * Create an evaluation model from an dom node
	 * 
	 * @param node
	 * @return
	 * @throws Exception
	 */
	private static EvaluationModel fromDomNode(Node node) throws Exception {
		int type = node.getNodeType();
		EvaluationModel model = null;

		switch (type) {
		case Node.DOCUMENT_NODE: {
			throw new Exception("Bad model file format:" + node.toString());
		}
		case Node.ELEMENT_NODE: {

			String name = node.getNodeName();
			if (name.equalsIgnoreCase(subEvalStr)) {
				model = new SubEvaluationModel();
			} else if (name.equalsIgnoreCase(objEvalStr)) {
				model = new ObjEvaluationModel();
			} else {
				throw new Exception("Bad model file format:" + name);
			}

			if (node.hasChildNodes()) {
				NodeList children = node.getChildNodes();
				for (int i = 0; i < children.getLength(); i++) {
					try {
						model.updateFromDomNode(children.item(i));
					} catch (Exception e) {
						e.printStackTrace();
					}
				}

			}
		}
		}
		return model;
	}


	protected abstract void updateFromDomNode(Node node) throws Exception;

	public static void showInstruction(String str) {
		String newStr = "";
		int maxLineLen = 100;
		while (str.length() > 0) {
			int lineLen = Math.min(maxLineLen, str.length());
			String curLine = str.substring(0, lineLen);
			int index = curLine.lastIndexOf(" ");
			if (index != -1 && str.length() > maxLineLen) {
				lineLen = index + 1;
			}
			index = curLine.lastIndexOf("\n");
			if (index != -1) {
				lineLen = index + 1;
			}
			newStr += str.substring(0, lineLen);
			// System.out.println("----" + str.substring(0, lineLen));
			str = str.substring(lineLen, str.length());
			newStr += "\n";
		}

		JOptionPane.showMessageDialog(null, newStr, "Instruction",
				JOptionPane.INFORMATION_MESSAGE);
	}

	public static Object[] getModelNames(List models) {
		ArrayList names = new ArrayList();
		for (int i = 0; i < models.size(); i++) {
			names.add(((EvaluationModel)models.get(i)).getName());
		}
		return names.toArray();
	}
	

	
}
