package evaluation.ui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.util.Arrays;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import evaluation.Judge;
import evaluation.models.ObjEvaluationModel;
import evaluation.utils.PicFilter;

public class ObjEvaluationWindow extends JFrame implements ActionListener,
		WindowListener {

	// The string shown in the next button
	private static String nextString = "Next";


	// The display time of a presentation
	private int displayTime = 10;

	private HashSet selectedFiles = new HashSet();

	private Judge judge = null;

	// The number of files already presented to user
	private int selectedFileNum = 0;

	// The number of total files
	private int totalFileNum = 0;

	private Button nextButton = new Button(nextString);

	private ObjQuestionPanel curQuestion;

	private Timer timer = new Timer();

	private ObjEvaluationModel model = null;
	// The list of presentation files
	private LinkedList fileList = null;
	
	// Whether questions are being showed at this time
	private boolean showingQuestion = false;

	public ObjEvaluationWindow(ObjEvaluationModel model) throws Exception {
		judge = model.getJudge();
		this.model = model;
		displayTime = model.getDisplayTime();
		File path = new File(model.getDataPath());
		// The list of all the sub directories in the data path
		File[] files = path.listFiles(new PicFilter());
		if (files == null) {
			System.out.println("Fail to get evaluation files.");
			System.exit(-1);
		}
		Arrays.sort(files);
		// Each element in the list is a list of files

		fileList = new LinkedList();
		for (int j = 0; j < files.length; j++) {
			fileList.add(files[j]);
			totalFileNum++;
		}

		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		nextButton.addActionListener(this);
		File curFile = selectFile(fileList, judge);
		showPresentation(curFile);
		addWindowListener(this);
	}

	/**
	 * Select a presentation file randomly
	 */
	private File selectFile(LinkedList files, Judge judge) {
		if (selectedFileNum >= totalFileNum) {
			return null;
		}
		Random ran = new Random();
		File result;
		int fileNum;
		boolean end = false;
		do {

			// Random choose a file
			int totalFileNum = files.size();
			fileNum = ran.nextInt(totalFileNum);
			result = (File) files.get(fileNum);
			if (selectedFiles.contains(result)) {
				end = false;
			} else {
				end = true;
				selectedFiles.add(result);
			}
		} while (!end);
		// Set the evaluationset number and presentation number of current judge
		judge.setCurPresentationNumber(fileNum);
		judge.setCurPresentationName(result.getName());
		selectedFileNum++;
		return result;
	}

	private void showPresentation(File file) throws Exception {
		getContentPane().removeAll();
		showingQuestion = false;
		// Add the image panel
		ImagePanel curImage = new ImagePanel(file);
		// System.out.println("******" + file);
		getContentPane().add(curImage);

		
		setVisible(true);
		setExtendedState(Frame.MAXIMIZED_BOTH);
		getContentPane().validate();
		startTimer(displayTime);
	}

	private void startTimer(int seconds) {
		timer.schedule(new TimerTask() {
			public void run() {
				showQuestions();
			}
		}, seconds * 1000);
	}

	private void showQuestions() {
		showingQuestion = true;
		getContentPane().removeAll();
		curQuestion = new ObjQuestionPanel(model);
		getContentPane().add(curQuestion);
		getContentPane().add(nextButton, BorderLayout.SOUTH);
		getContentPane().validate();
		setVisible(true);
		setExtendedState(Frame.MAXIMIZED_BOTH);
	}

	public void actionPerformed(ActionEvent e) {
		try {
			// System.out.println("*********" + e.toString());
			String command = e.getActionCommand();
			if (command.equalsIgnoreCase(nextString)) {
				checkAnswers();
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Check whether all the questions were answered.
	 */
	private void checkAnswers() throws Exception {
		if (!curQuestion.allAnswered()) {
			JOptionPane.showMessageDialog(null,
					"You have to answer all questions.", "Error",
					JOptionPane.ERROR_MESSAGE);
		} else {
			File curFile = selectFile(fileList, judge);
			if (curFile == null) {
				dispose();
				model.endEvaluation(fileList);
			} else {
				showPresentation(curFile);
			}
		}

	}




	public void windowClosing(WindowEvent e) {
		try {
			if (showingQuestion) {
				checkAnswers();
			} else {
				timer.cancel();
				dispose();
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public void windowClosed(WindowEvent e) {

	}

	public void windowOpened(WindowEvent e) {

	}

	public void windowIconified(WindowEvent e) {

	}

	public void windowDeiconified(WindowEvent e) {

	}

	public void windowActivated(WindowEvent e) {

	}

	public void windowDeactivated(WindowEvent e) {

	}

	public void windowGainedFocus(WindowEvent e) {

	}

	public void windowLostFocus(WindowEvent e) {

	}

	public void windowStateChanged(WindowEvent e) {

	}

}
