package evaluation.ui;

import java.awt.GridLayout;
import java.awt.Label;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.Border;

import evaluation.Answer;
import evaluation.Judge;
import evaluation.Question;
import evaluation.models.ObjEvaluationModel;

class ObjQuestionPanel extends JPanel implements ActionListener {
	private static String remarkString = "Please answer the following questions according to the previous presentation:";

	public static String splitString = "@";

	private Judge judge = null;

	// The button groups for the questions
	private ButtonGroup[] groups;

	private LinkedList questions;

	public ObjQuestionPanel(ObjEvaluationModel model) {
		judge = model.getJudge();
		questions = model.getCurQuestions();
		
		groups = new ButtonGroup[questions.size()];
		setLayout(new GridLayout(questions.size() + 1, 1));
		// Add the question text
		Label remark = new Label(remarkString);
		add(remark);
		// Add questions
		for (int i = 0; i < questions.size(); i++) {
			groups[i] = new ButtonGroup();
			addQuestion(i);
		}
	}

	private void addQuestion(int questionNum) {
		ButtonGroup group = groups[questionNum];
		JPanel optionPanel = new JPanel();
		Border etched = BorderFactory.createEtchedBorder();
		Border titled = BorderFactory.createTitledBorder(etched, "Statement "
				+ (questionNum + 1));
		// Find the answer for current question
		Answer curAnswer = judge.getAnswerForQuestion(questionNum);
		int answerNum = -1;
		if (curAnswer != null) {
			answerNum = curAnswer.getQuestionAnswer();
		}

		optionPanel.setBorder(titled);
		Question question = (Question) questions.get(questionNum);
		// Add the question text
		Label text = new Label(question.getQuestionText(), Label.LEFT);
		optionPanel.add(text);
		// Add the question options
		LinkedList optionTexts = question.getOptionTexts();
		JRadioButton button;
		for (int i = 0; i < optionTexts.size(); i++) {
			button = new JRadioButton(optionTexts.get(i).toString());
			button.setActionCommand(questionNum + splitString + i);
			optionPanel.add(button);
			group.add(button);
			button.addActionListener(this);
			if (i == answerNum) {
				button.setSelected(true);
			}
		}
		add(optionPanel);
	}

	public void actionPerformed(ActionEvent e) {
		try {
			String command = e.getActionCommand();

			if (isSelection(command)) {
				judge.addAnswer(command, questions);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Check whether the command is the selection of an option
	 */
	private boolean isSelection(String command) {
		if (command.indexOf(splitString) != 0) {
			return true;
		} else {
			return false;
		}
	}

	public boolean allAnswered() {
		for (int i = 0; i < groups.length; i++) {
			ButtonModel model = groups[i].getSelection();
			if (model == null) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Get the number of answered questions.
	 */
	public int getAnsweredNum() {
		int result = 0;
		for (int i = 0; i < groups.length; i++) {
			ButtonModel model = groups[i].getSelection();
			if (model != null) {
				result++;
			}
		}
		return result;
	}
}
