package evaluation.ui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.util.Arrays;
import java.util.LinkedList;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import evaluation.Evaluation;
import evaluation.Judge;
import evaluation.models.SubEvaluationModel;
import evaluation.utils.PicFilter;

public class SubEvaluationWindow extends JFrame implements ActionListener,
		WindowListener {
	// The string shown in the button
	private static String buttonPreStr = "File";

	private LinkedList fileList;

	ImageButton[] buttons;

	SubEvaluationModel model;

	// The number of selected files
	int selectedNum = 0;

	public SubEvaluationWindow(SubEvaluationModel model) throws Exception {
		String title = "Screen " + Evaluation.getCurSetNum() + "/"
				+ Evaluation.getTotalSetsNum();
		setTitle(title);
		this.model = model;
		selectedNum = 0;
		File path = new File(model.getDataPath());
		// The list of all the sub directories in the data path
		File[] files = path.listFiles(new PicFilter());
		if (files == null) {
			System.out.println("Fail to get evaluation files.");
			System.exit(-1);
		}
		Arrays.sort(files);
		// Each element in the list is a list of files

		fileList = new LinkedList();
		for (int j = 0; j < files.length; j++) {
			fileList.add(files[j]);
		}
		JPanel topPanel = new JPanel();
		// Label text = new Label("Please click the button to view the image.");
		JLabel text = new JLabel(
				"Please click on the presentation to view it. When you have viewed all presentations, close this window to move on.");
		text.setFont(new Font("Serif", Font.BOLD, 14));
		topPanel.add(text, 0);
		getContentPane().add(topPanel, BorderLayout.NORTH);
		JPanel centerPanel = new JPanel();
		buttons = new ImageButton[fileList.size()];
		int rowNum = (int) Math.sqrt(fileList.size());
		if (rowNum < Math.sqrt(fileList.size())) {
			rowNum++;
		}
		int columnNum = rowNum;
		centerPanel.setLayout(new GridLayout(columnNum, rowNum, 10, 10));
		for (int i = 0; i < buttons.length; i++) {
			buttons[i] = new ImageButton((File) fileList.get(i), buttonPreStr + i);
			centerPanel.add(buttons[i]);
			buttons[i].addActionListener(this);
		}
		centerPanel.validate();
		getContentPane().add(centerPanel);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		setVisible(true);
		setExtendedState(Frame.MAXIMIZED_BOTH);
		addWindowListener(this);
	}

	public void actionPerformed(ActionEvent e) {
		try {
			// System.out.println("*********" + e.toString());
			String command = e.getActionCommand();
			if (isFileString(command)) {
				int num = getFileNum(command);
				Judge judge = model.getJudge();
				judge.setCurPresentationNumber(num);
				buttons[num].setSelected();
				SubEvaluationBrowserWindow bw = new SubEvaluationBrowserWindow(
						(File) fileList.get(num), model);
				selectedNum++;
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private boolean isFileString(String str) {
		if (str.indexOf(buttonPreStr) == 0) {
			return true;
		} else {
			return false;
		}
	}

	private int getFileNum(String str) {
		String numStr = str.substring(buttonPreStr.length(), str.length());
		Integer num = new Integer(numStr);
		return num.intValue();
	}

	public void windowClosing(WindowEvent e) {
		if (fileList.size() > selectedNum) {
			// JOptionPane.showMessageDialog(null, "You must view all the
			// presentations before you close the window.", "Error",
			// JOptionPane.ERROR_MESSAGE);
		}
		// else {
		dispose();
		model.endEvaluation(fileList);
		// }
	}

	public void windowClosed(WindowEvent e) {

	}

	public void windowOpened(WindowEvent e) {

	}

	public void windowIconified(WindowEvent e) {

	}

	public void windowDeiconified(WindowEvent e) {

	}

	public void windowActivated(WindowEvent e) {

	}

	public void windowDeactivated(WindowEvent e) {

	}

	public void windowGainedFocus(WindowEvent e) {

	}

	public void windowLostFocus(WindowEvent e) {

	}

	public void windowStateChanged(WindowEvent e) {

	}

}
